txbufferP equ 7000h ;the location of 256 byte circular buffer
txBufHeadP equ 7100h ;pointer to byte that stores head pointer
txBufTailP equ 7101h ;pointer to byte that stores tail pointer
txBufSizeP equ 7102h ;pointer to byte that stores number of bytes in buffer
orgP equ 6000h ;the origin of the program
kbtableP equ 6400h ;location of keyboard lookup table

org orgP
ld sp,06fffh

start:
ld a,0
ld (txBufHeadP),a
ld (txBuftailP),a
ld (txBufSizeP),a ;zero the circular buffer

setup232:
out (0e8h),a ;reset the uart
ld a,000h
out (0e9h),a ;set baud to 9600
ld a,11101100b ;set 8n1, and rts and dts
out (0eah),a

mloop:
call txData
push bc
call kbscan
pop bc
ld a,d
sub b
jp z,mloop ;lock into loop until a different key is pressed

ld a,d
sub 0ffh
jp z, skpo ;if there are unprintable characters, skip it
ld a,d

call txBufPut

skpo:
ld b,d ;b now has last pressed key
jp mloop


txData: ;get data from the tx circular buffer (if any) and send it out
in a,(0eah)
bit 6,a
ret z
call txBufGet
ret z
;if we get here, there is data to be sent
out (0ebh),a
ld (3c00h),a
ret

txDataB:


txBufPut: ;add byte (a) into the circular buffer. Uses a,b,hl
ld b,a
ld a,(txBufSizeP)
sub 254
ret z ;check to see if the buffer is full. if it is, return immediately (zero flag will be set)
add a,255 
ld (txBufSizeP),a ;increment txbufSize by 1 and save
ld hl,txbufferP
ld a,(txBufHeadP)
inc a
ld (txBufHeadP),a
ld l,a ;get save and load a new value for the buffer head pointer
ld (hl),b ;save the byte into the circular buffer
and a ;clear zero buffer
ret

txBufGet: ;gets a byte from the circular buffer. Uses a,hl
ld a,(txBufSizeP)
sub 0
ret z ;check to see if the buffer is empty. if it is, return immediately (zero flag will be set)
sub 1
ld (txBufSizeP),a ;decrement txbufSize by 1 and save
ld hl,txbufferP
ld a,(txBufTailP)
inc a
ld (txBufTailP),a
ld l,a ;get save and load a new value for the buffer tail pointer
and a ;clear zero buffer
ld a,(hl) ;get the byte from the circular buffer
ret



kbscan: ;scans keyboard. Returns d, either scan code or ffh if no key pressed
ld hl,3801h
ld c,255
ld d,0
KBreadA:
ld e,(hl)
ld a,08h
sub l
jp z, KBreadB
KbreadC:
ld b,0
KBreadrowB:
ld a,1
and e
jp nz, KBreadrowC
KBreadrowD:
rr e
inc d
ld a,7
cp b
jp z,kbreadrowE
inc b
jp kbreadrowB
kbreadrowE:
ld a,20H
sub l
jp z, KBreadX
rl l
jp KBreadA
KBreadB: ;add in arrows and space into row 08
ld a,(3840h)
and 0f8h
add a,e
ld e,a
jp KBreadC
KBreadrowC: ;if a key is found pressed, store that index
ld a,d
ld c,a
jp KBreadrowD
KBreadX: ;done scanning most keys, check modifiers
ld d,c
ld a,255
sub a,c
jp z,nnKey ;no (normal) key was pressed

ld a,(3840H) ;check control key (clear)
and 2
sub 2
jp nz,nClear
ld a,96
add a,c
ld c,a
nClear:

ld a,(3880H) ;check shift key
and 1
jp z,nShift
ld a,48
add a,c
ld c,a
nShift:

;get value from lookup table
ld hl,kbtableP
ld l,c
ld d,(hl)

nnKey:
ld a,(3840H) ;check enter/break keys
ld b,a
and 1
jp z,nEnter
ld a,0FEh
ld d,a
nEnter:
ld a,b
and 4
sub 4
jp nz,nBreak
ld a,0F0h
ld d,a
nBreak:

ret

seek kbtableP-orgP
incbin "kbmap.bin"