orgP        equ 6000h ;the origin of the program
txbufferP   equ 7100h ;the location of 256 byte circular buffer
txBufHeadP  equ 7000h ;pointer to byte that stores head pointer
txBufTailP  equ 7001h ;pointer to byte that stores tail pointer
txBufSizeP  equ 7002h ;pointer to byte that stores number of bytes in buffer

rxbufferP   equ 7200h ;the location of 256 byte circular buffer
rxBufHeadP  equ 7003h ;pointer to byte that stores head pointer
rxBufTailP  equ 7004h ;pointer to byte that stores tail pointer
rxBufSizeP  equ 7005h ;pointer to byte that stores number of bytes in buffer

lastKeyP    equ 7007h ;pointer to byte that stores the last pressed key
kbtableP    equ 6400h ;location of keyboard lookup table
cursorCharP equ 7010h ;pointer to value of byte stored at cursor position
cursorRowP  equ 7011h ;pointer to cursor row position
cursorColP  equ 7012h ;pointer to cursor col position

org orgP
ld sp,0ffffh

;screencopy: ;fill with some sample text
;ld bc,1024
;ld de,3c00h
;ld hl,7800h
;ldir 

start:
ld a,0
ld (cursorRowP),a
ld (cursorColP),a
ld (rxBufHeadP),a
ld (rxBufTailP),a
ld (rxBufSizeP),a
ld (txBufHeadP),a
ld (txBufTailP),a
ld (txBufSizeP),a ;zero the circular buffers
ld a,0ffh
ld (lastKeyP),a   ;assume last key pressed was no key

call screenClear

setup232:
out (0e8h),a ;reset the uart
ld a,0eeh
out (0e9h),a ;set baud to 9600
ld a,11101100b ;set 8n1, and rts and dts
out (0eah),a

mloop:
call txData ;if there is data to be sent, try to send it

call kbNewKey ;look for new keypresses
jp z, newKeyEnd
;there is a new key if we get here
call txBufPut
newKeyEnd: 

;call rxData

call rxBufGet
jp z, endRXm ;buffer is empty
cp 20h
jp c, nprintable
cp 127
jp nc, nprintable
ld (cursorCharP),a ;normal printable characters display on screen
call cursorRight
jp endRxm
nprintable: ;handle non-printable characters here
ld bc,endRXm
push bc ;return address for calls is the end of this loop
cp 0ah
jp z, cursorLF
cp 0Dh
jp z, cursorCR
cp 1bh
jp nz,endNonPrintable
;handle escape codes here
ld a,1
call rxDataIn
call rxBufGet
cp 65; A
jp z, cursorUp
cp 66; B
jp z, cursorDown
cp 67; C
jp z, cursorRight
cp 68; D
jp z, cursorLeft
cp 72; H
jp z, cursorHome
cp 74; J
jp z, screenCleartoEnd
cp 75; K
jp z, screenCleartoEndLine
cp 78; N - clear screen (non-standard)
jp z, screenClear
cp 89; Y - move cursor
jp nz, endNonPrintable
ld a,2 ;need to get 2 more bytes for cursor placement
call rxDataIn
call rxBufGet
sub 32
ld l,a
push hl
call rxBufGet
sub 32
pop hl
ld h,a
call cursorMove
endNonPrintable:
pop bc ;pop address back if we went nowhere
endRXm:
jp mloop

rxDataIn: ;loads data into the rx buffer until buffer size is a or above
push bc
ld b,a
rxDataInB:

;push bc
;call kbNewKey ;look for new keypresses
;jp z, newKeyEnd2
;there is a new key if we get here
;call rxBufPut
;newKeyEnd2: 
;pop bc

call rxData
ld a,(rxBufSizeP)
cp b 
jp c, rxDataInB
pop bc
ret

rxData: ;if there is data to input, store it into the rx buffer
;ret ;temp disable
push af
push bc
push hl
in a,(0eah)
bit 7,a
jp z,rxDataB ;retun if there is nothing for us
in a,(0ebh)
call rxBufPut
rxDataB:
pop hl
pop bc
pop af
ret

txData: ;get data from the tx circular buffer (if any) and send it out
in a,(0eah)
bit 6,a
ret z
call txBufGet
ret z
;if we get here, there is data to be sent
out (0ebh),a
ret

txBufPut: ;add byte (a) into the circular buffer. Uses a,b,hl
ld b,a
ld a,(txBufSizeP)
sub 254
ret z ;check to see if the buffer is full. if it is, return immediately (zero flag will be set)
add a,255 
ld (txBufSizeP),a ;increment txbufSize by 1 and save
ld hl,txbufferP
ld a,(txBufHeadP)
inc a
ld (txBufHeadP),a
ld l,a ;get save and load a new value for the buffer head pointer
ld (hl),b ;save the byte into the circular buffer
and a ;clear zero buffer
ret

txBufGet: ;gets a byte from the circular buffer. Uses a,hl
ld a,(txBufSizeP)
sub 0
ret z ;check to see if the buffer is empty. if it is, return immediately (zero flag will be set)
sub 1
ld (txBufSizeP),a ;decrement txbufSize by 1 and save
ld hl,txbufferP
ld a,(txBufTailP)
inc a
ld (txBufTailP),a
ld l,a ;get save and load a new value for the buffer tail pointer
and a ;clear zero buffer
ld a,(hl) ;get the byte from the circular buffer
ret

rxBufPut: ;add byte (a) into the circular buffer. Uses a,b,hl
ld b,a
ld a,(rxBufSizeP)
sub 254
ret z ;check to see if the buffer is full. if it is, return immediately (zero flag will be set)
add a,255 
ld (rxBufSizeP),a ;increment txbufSize by 1 and save
ld hl,rxbufferP
ld a,(rxBufHeadP)
inc a
ld (rxBufHeadP),a
ld l,a ;get save and load a new value for the buffer head pointer
ld (hl),b ;save the byte into the circular buffer

and a ;clear zero buffer
ret

rxBufGet: ;gets a byte from the circular buffer. Uses a,hl
ld a,(rxBufSizeP)
sub 0
ret z ;check to see if the buffer is empty. if it is, return immediately (zero flag will be set)
sub 1
ld (rxBufSizeP),a ;decrement txbufSize by 1 and save
ld hl,rxbufferP
ld a,(rxBufTailP)
inc a
ld (rxBufTailP),a
ld l,a ;get save and load a new value for the buffer tail pointer
and a ;clear zero buffer
ld a,(hl) ;get the byte from the circular buffer
ret

kbNewKey: ;returns with a, zero flag clear if a different key has just been pressed
push bc
call kbScan
ld b,a
ld a,(lastKeyP)
cp b
jp z, notnewkey
ld a,b
ld (lastKeyP),a
ld b,0ffh
cp b
notnewkey:
pop bc
ret

kbScan: ;scans keyboard. Returns a - keyboard scan code.
;most keys (and shift) work how you expect
;clear key used as a control key (for ascii control characters and other special symbols)
;enter and break keys are simgular, and have special scancodes (shifted arrows also have special codes)
push bc
push de
push hl
ld hl,3801h
ld c,255
ld d,0
KBreadA:
ld e,(hl)
ld a,08h
sub l
jp z, KBreadB
KbreadC:
ld b,0
KBreadrowB:
ld a,1
and e
jp nz, KBreadrowC
KBreadrowD:
call rxData
rr e
inc d
ld a,7
cp b
jp z,kbreadrowE
inc b
jp kbreadrowB
kbreadrowE:
ld a,20H
sub l
jp z, KBreadX
rl l
jp KBreadA
KBreadB: ;add in arrows and space into row 08
ld a,(3840h)
and 0f8h
add a,e
ld e,a
jp KBreadC
KBreadrowC: ;if a key is found pressed, store that index
ld a,d
ld c,a
jp KBreadrowD
KBreadX: ;done scanning most keys, check modifiers
ld d,c
ld a,255
sub a,c
jp z,nnKey ;no (normal) key was pressed
ld a,(3840H) ;check control key (clear)
and 2
sub 2
jp nz,nClear
ld a,96
add a,c
ld c,a
nClear:
ld a,(3880H) ;check shift key
and 1
jp z,nShift
ld a,48
add a,c
ld c,a
nShift:
;get value from lookup table
ld hl,kbtableP
ld l,c
ld d,(hl)
nnKey:
ld a,(3840H) ;check enter/break keys
ld b,a
and 1
jp z,nEnter
ld a,13
ld d,a
nEnter:
ld a,b
and 4
sub 4
jp nz,nBreak
ld a,0EFh
ld d,a
nBreak:
;ld a,0ffh
;sub d
ld a,d
;jp z, nokeys
;nokeys:
pop hl
pop de
pop bc
ret

screenScroll: ;scrolls screen up. saves everything
push af
push bc
push de
push hl
ld a,15
ld de, 3c00h
ld hl, 3c40h
screenScrollB:
ld bc, 64
screenScrollC:
call rxData
ldi
jp pe,screenScrollC
dec a
jp nz,screenScrollB
ld b,64
screenScrollD:
ld a,20h
ld (de),a
inc de
dec b
jp nz, screenScrollD
pop hl
pop de
pop bc
pop af
ret

cursorP: ;sets HL with screen address of row/col. Call with h as col, l as row. Uses F
push de
push bc
ld d,0
ld e,h
ld h,0
add hl,hl
add hl,hl
add hl,hl
add hl,hl
add hl,hl
add hl,hl
add hl,de
ld de, 3c00h
add hl,de
ld d,h
ld e,l
ld bc, 3fffh
sbc hl,bc
jp c,cursorPC
ld h,b
ld l,c
jp cursorPD
cursorPC:
ld h,d
ld l,e
cursorPD:
pop bc
pop de
ret

cursorPlace: ;Writes cursor character to l/h position, and saves value of whatever used to be there
ld b,h
ld c,l
call cursorP
ld a,(hl)
ld (cursorCharP),a
ld a,8fh
ld (hl),a
ld a,b
ld (cursorColP),a
ld a,c
ld (cursorRowP),a
ld a,0
cp 0
ret

cursorUnPlace: ;Writes the cursor character back over the cursor
ld a,(cursorColP)
ld h,a
ld a,(cursorRowP)
ld l,a
ld a,(cursorCharP)
call cursorP
ld (hl),a
ld a,0
cp 0
ret

cursorMove: ;writes back last cursor position with whatever was there before, and puts cursor at new position
push bc
ld b,h
ld c,l
ld a,(cursorColP)
ld h,a
ld a,(cursorRowP)
ld l,a
call cursorP
ld a,(cursorCharP)
ld (hl),a
ld h,b
ld l,c
call cursorPlace
pop bc
ld a,0
cp 0
ret

cursorRight:
ld a,(cursorRowP)
ld b,a
ld a,(cursorColP)
inc a
cp 64
jp nz, cursorRightB
ld a,b
inc a
cp 16
jp nz, cursorRightC
;get here, need to scroll screen up and put cursor at left
ld a,(cursorCharP) 
ld (3FFFh),a ;write back character
ld a,20h
ld (cursorCharP),a
call screenScroll
ld a,0
ld (cursorColP),a
ld a,15
ld (cursorRowP),a
ld a,8fh
ld (3fc0h),a
ret
jp cursorRightB
cursorRightC:
ld b,a
ld a,0
cursorRightB:
ld h,a
ld l,b
call cursorMove
ld a,0
cp 0
ret

cursorLeft:
ld a,(cursorRowP)
ld b,a
ld a,(cursorColP)
dec a
cp 255
jp nz, cursorLeftB
ld a,b
dec a
cp 255
jp nz, cursorLeftC
ld a,0
ld b,0
jp cursorLeftB
cursorLeftC:
ld b,a
ld a,63
cursorLeftB:
ld h,a
ld l,b
call cursorMove
ld a,0
cp 0
ret

cursorDown:
ld a,(cursorRowP)
inc a
cp 16
jp nz, cursorDownB
ld a,0
cursorDownB:
ld l,a
ld a,(cursorColP)
ld h,a
call cursorMove
ld a,0
cp 0
ret

cursorUp:
ld a,(cursorRowP)
dec a
cp 255
jp nz, cursorUpB
ld a,15
cursorUpB:
ld l,a
ld a,(cursorColP)
ld h,a
call cursorMove
ld a,0
cp 0
ret

cursorLF:
ld a,(cursorRowP)
cp 15
jp nz, cursorLFB
call cursorUnPlace
call screenScroll
ld a,(cursorColP)
ld h,a
ld l,15
call cursorPlace
jp cursorLFC
cursorLFB:
call cursorDown
cursorLFC:
ld a,0
cp 0
ret

cursorCR:
ld a,(cursorRowP)
ld l,a
ld h,0
call cursorMove
ret

cursorCRLF:
call cursorCR
call cursorLF
ret

cursorHome:
ld h,0
ld l,0
call cursorMove
ret

screenClear:
call cursorHome
call screenCleartoEnd
ret

screenCleartoEnd:
push af
push bc
push hl
ld a,(cursorRowP)
ld l,a
ld a,(cursorColP)
ld h,a
call cursorP
push hl
ld bc,4000h
screenCleartoEndB:
call rxData
ld a,20h
ld (hl),a
or a
inc hl
sbc hl,bc
add hl,bc
jp nz,screenCleartoEndB
pop hl
ld a,8fh
ld (hl),a
ld a,20h
ld (cursorCharP),a
pop hl
pop bc
pop af
ret

screenCleartoEndLine:
ld a,(cursorRowP)
ld l,a
ld a,(cursorColP)
ld h,a
ld b,a
call cursorP
ld a,20h
ld c,a
screenCleartoEndLineB:
inc b
ld a,b
cp 64
jp z, screenCleartoEndLineC 
inc hl
ld (hl),c
jp screenCleartoEndLineB
screenCleartoEndLineC:
ld a,20h
ld (cursorCharP),a
ret


hexdisp: ;writes to screen hex value of a
push af
push bc
ld c,a
and 11110000b
rra
rra
rra
rra
ld b,a
ld a,c
and 00001111b
add a,30h
cp 3ah
jp c,notletterF
add a,7h
notletterF:
ld (3c3fh),a
ld a,b
add a,30h
cp 3ah
jp c,notletterG
add a,7h
notletterG:
ld (3c3eh),a
pop bc
pop af
ret

seek kbtableP-orgP
incbin "kbmap.bin"

;seek 1800h
;incbin "screentest.bin"